package com.jiuqi.mde;

import com.jiuqi.util.NumberFormatManager;
import java.util.HashMap;
import java.util.Map;
import java.util.ArrayList;

public class MDEDimensionSet implements Cloneable, Comparable
{
    private String dimensionNames;
    private ArrayList dimensions;
    private Map nameSearch;
    public static final int SCR_EQUAL = 0;
    public static final int SCR_MORE = 1;
    public static final int SCR_LESS = 2;
    public static final int SCR_LESS_AND_MORE = 3;
    
    public MDEDimensionSet() {
        this.dimensionNames = null;
        this.dimensions = new ArrayList(2);
        this.nameSearch = new HashMap(10);
    }
    
    public final int size() {
        return this.dimensions.size();
    }
    
    public final MDEDimension get(final int index) {
        return this.dimensions.get(index);
    }
    
    public final MDEDimension find(final String name) {
        final Integer index = this.nameSearch.get(name);
        return (index != null) ? this.dimensions.get(index) : null;
    }
    
    public final void clear() {
        this.dimensions.clear();
        this.nameSearch.clear();
        this.dimensionNames = null;
    }
    
    private final int findOrder(final String order) {
        int start = 0;
        int stop = this.dimensions.size() - 1;
        int flag = -1;
        while (start <= stop) {
            final int middle = start + stop + 1 >> 1;
            flag = order.compareTo(this.dimensions.get(middle).order);
            if (flag > 0) {
                start = middle + 1;
            }
            else if (flag < 0) {
                stop = middle - 1;
            }
            else {
                start = middle;
                if (start == stop) {
                    break;
                }
                continue;
            }
        }
        return (flag == 0) ? start : (-start - 1);
    }
    
    public final void add(final MDEDimension dimension) {
        if (!this.nameSearch.containsKey(dimension.name)) {
            int index = this.findOrder(dimension.order);
            if (index < 0) {
                index = -1 - index;
            }
            else {
                ++index;
            }
            this.dimensionNames = null;
            this.dimensions.add(index, dimension);
            for (int i = index, count = this.dimensions.size(); i < count; ++i) {
                this.nameSearch.put(this.dimensions.get(i).name, NumberFormatManager.getInt(i));
            }
        }
    }
    
    public final void add(final MDEDimensionSet dimensions) {
        for (int i = 0, count = dimensions.size(); i < count; ++i) {
            this.add(dimensions.get(i));
        }
    }
    
    public final void remove(final MDEDimension dimension) {
        final Integer index = this.nameSearch.get(dimension.name);
        if (index != null) {
            this.dimensionNames = null;
            this.nameSearch.remove(dimension.name);
            this.dimensions.remove((int)index);
            for (int i = index, count = this.dimensions.size(); i < count; ++i) {
                this.nameSearch.put(this.dimensions.get(i).name, NumberFormatManager.getInt(i));
            }
        }
    }
    
    public final int indexOf(final MDEDimension dimension) {
        final Integer index = this.nameSearch.get(dimension.name);
        return (index != null) ? index : -1;
    }
    
    public final int indexOf(final String name) {
        final Integer index = this.nameSearch.get(name);
        return (index != null) ? index : -1;
    }
    
    public final boolean contains(final MDEDimension dimension) {
        return this.nameSearch.get(dimension.name) != null;
    }
    
    public final boolean contains(final String name) {
        return this.nameSearch.get(name) != null;
    }
    
    public final MDEDimension findByFieldName(final String fieldName) {
        for (int i = 0, count = this.dimensions.size(); i < count; ++i) {
            final MDEDimension dimension = this.dimensions.get(i);
            if (fieldName.equals(dimension.fieldName)) {
                return dimension;
            }
        }
        return null;
    }
    
    public final boolean hasOrder() {
        return this.find("顺序号") != null;
    }
    
    public Object clone() {
        final MDEDimensionSet result = new MDEDimensionSet();
        result.add(this);
        return result;
    }
    
    public final int compareTo(final Object o) {
        if (this == o) {
            return 0;
        }
        if (o == null) {
            return 1;
        }
        if (this == null) {
            return -1;
        }
        if (o instanceof MDEDimensionSet) {
            return this.toString().compareTo(((MDEDimensionSet)o).toString());
        }
        return this.hashCode() - o.hashCode();
    }
    
    @Override
    public final boolean equals(final Object o) {
        return this.compareTo(o) == 0;
    }
    
    public final int collectCompare(final MDEDimensionSet another) {
        if (this == another) {
            return 0;
        }
        if (another == null) {
            return 1;
        }
        if (this == null) {
            return -1;
        }
        int result = 0;
        int matched = 0;
        for (int i = 0, count = this.dimensions.size(); i < count; ++i) {
            final MDEDimension dimension = this.dimensions.get(i);
            if (another.contains(dimension)) {
                ++matched;
            }
            else {
                result |= 0x1;
            }
        }
        if (matched < another.size()) {
            result |= 0x2;
        }
        return result;
    }
    
    public final boolean contains(final MDEDimensionSet another) {
        final int cr = this.collectCompare(another);
        return cr == 0 || cr == 1;
    }
    
    public boolean lessAndMore(final MDEDimensionSet another) {
        final int cr = this.collectCompare(another);
        return cr == 3;
    }
    
    @Override
    public final String toString() {
        if (this.dimensionNames == null) {
            final StringBuffer result = new StringBuffer(64);
            final boolean needComma = false;
            for (int i = 0, count = this.dimensions.size(); i < count; ++i) {
                final MDEDimension dimension = this.dimensions.get(i);
                if (needComma) {
                    result.append(", ");
                }
                result.append(dimension.name);
            }
            this.dimensionNames = result.toString();
        }
        return this.dimensionNames;
    }
}
